Resid.mmhp <- function(ti, lamb0, nu0, eta0, Q0, pai0, fortran = TRUE)
{ 
    library(Matrix)
    n <- length(ti) - 1
    r <- length(lamb0)
    xi <- ti[2:(n + 1)] - ti[1:n]
    lamb <- lamb0
    nu <- nu0
    eta <- eta0
    Q <- Q0
    pai <- pai0
    sums <- matrix(as.double(0), n, r)
    lamL <- matrix(as.double(0), n, r)
    LamU <- array(as.double(0), dim = c(n, r, r))
    lamstar <- .Fortran("lamstar", as.integer(r), as.integer(n), 
        as.double(lamb), as.double(nu), as.double(eta), as.double(ti), 
        sums, lamL, LamU, PACKAGE = "MMHP")
    names(lamstar) <- c("r", "n", "lamb", "nu", "eta", "ti", 
        "sums", "lams", "Lamst")
    lams <- lamstar$lams
    Lamst <- lamstar$Lamst
    S <- array(as.double(0), dim = c(n, r, r))
    Sinv <- array(as.double(0), dim = c(n, r, r))
    post <- array(as.double(0), dim = c(n, r, r))
    eigenval <- matrix(as.double(0), n, r)
    for (i in 1:n) {
        decomp <- eigen(Q - Lamst[i, , ], symmetric = FALSE)
        if (any(duplicated(decomp$values))) 
            stop("repeated eigenvalues")
        S[i, , ] <- decomp$vectors
        Sinv[i, , ] <- solve(S[i, , ])
        post[i, , ] <- Sinv[i, , ] %*% Lamst[i, , ]
        eigenval[i, ] <- decomp$values
    }
    fk <- array(as.double(0), dim = c(n, r, r))
    expmt <- array(as.double(0), dim = c(n, r, r))
    Ln <- matrix(as.double(0), n + 1, r)
    Lfk <- matrix(as.double(0), n, r)
    Lfksum <- rep(as.double(0), n)
    RR <- matrix(as.double(0), n + 1, r)
    Rfk <- matrix(as.double(0), n, r)
    tem <- matrix(as.double(0), n, r)
    C <- array(as.double(0), dim = c(n, 2 * r, 2 * r))
    Cfun <- .Fortran("Cfun", as.integer(r), as.integer(n), 
        fk, RR, Rfk, Ln, Lfk, Lfksum, S, Sinv, eigenval, 
        Lamst, post, as.double(xi), expmt, as.double(pai), 
        as.double(Q), tem, C, PACKAGE = "MMHP")
    names(Cfun) <- c("r", "n", "fk", "RR", "Rfk", "L", "Lfk", 
        "Lfksum", "S", "Sinv", "eigenval", "Lamst", "post", 
        "xi", "expmt", "pai", "Q", "tem", "C")
    C <- Cfun$C
    Lamst <- Cfun$Lamst
    L <- Cfun$L
    R <- Cfun$RR
    Ik <- array(as.double(0), dim = c(n, r, r))
    for (k in 1:n) {
        Ik[k, , ] <- (as.matrix(expm(Matrix(C[k, , ] * xi[k])))[1:r, 
            (r + 1):(2 * r)])
    }
    Icn <- matrix(as.double(0), r, r)
    wn <- matrix(as.double(0), r, r)
    Dn <- rep(as.double(0), r)
    wDfun <- .Fortran("wDfun", as.integer(r), as.integer(n), 
        fk, Ln, Lfk, Lfksum, S, Sinv, eigenval, Lamst, post, 
        as.double(xi), expmt, as.double(pai), Ik, Icn, as.double(Q), 
        wn, Dn, PACKAGE = "MMHP")
    names(wDfun) <- c("r", "n", "fk", "L", "Lfk", "Lfksum", 
        "S", "Sinv", "eigenval", "Lamst", "post", "xi", "expmt", 
        "pai", "Ik", "Ic", "Q", "w", "D")
    Ic <- wDfun$Ic
    Lfksum <- wDfun$Lfksum
    w <- wDfun$w
    D <- wDfun$D
    Bn <- matrix(as.double(0), r, n)
    Bfun <- .Fortran("Bfun", as.integer(r), as.integer(n), 
        fk, RR, Rfk, Ln, Lfk, Lfksum, S, Sinv, eigenval, 
        Lamst, post, as.double(xi), expmt, as.double(pai), 
        Bn, PACKAGE = "MMHP")
    names(Bfun) <- c("r", "n", "fk", "RR", "Rfk", "L", "Lfk", 
        "Lfksum", "S", "Sinv", "eigenval", "Lamst", "post", 
        "xi", "expmt", "pai", "Bn")
    B <- Bfun$Bn
    tis <- ti[-1]
    lx <- length(tis)
    estStat <- estInt <- matrix(0,r,lx)
    for (i in 1:r){
      estInt[i,] <- apply(t(tis), 2, 
        function(t, ti, lamb, nu, eta){
        data <- rbind(NULL, cbind(time = ti, magnitude = rep(0,n+1)))
        param <- as.matrix(rbind(lamb,nu,eta))
    #    HawkesStep.cif(data, t, params=param[,i])
        lamb[i]+nu[i]*eta[i]*
              sum(exp(-eta[i]* ( max(ti[ti<t]) - ti[ti<max(ti[ti<t])] )))
        },ti=ti,lamb=lamb, nu=nu, eta=eta)
    }
    Reside <- NULL
    for (k in 1:(n))
    {
     tem <- 0
     for (i in 1:r)
     {
      tem <- tem + sum(estInt[i,1:k] * Ik[1:k,i,i] / Lfksum[1:k])
     }
     Reside[k] <- tem
    }
    return(list(Resid=Reside))
}
