\name{mmhp}
\alias{mmhp}

\title{Parameter Estimation of a Markov-Modulated Hawkes Process with Stepwise Decay (MMHP).}
\description{
  Calculates the parameter estimates of an MMHP process.
}
\usage{
mmhp(ti, lamb0, nu0, eta0, Q0, pai0, nlmp = TRUE, fortran = TRUE)
}

\arguments{
  \item{ti}{is a vector containing the occurrence times of the observed process. Note that the first event is at time zero.}
  \item{lamb0}{is a vector containing the background rates of Hawkes processes with stepwise decay.}
  \item{nu0}{is a vector containing the parameter with values between 0 and 1.}
  \item{eta0}{is a vector containing the decay rates of Hawkes processes with stepwise decay.}
  \item{Q0}{is the infinitesimal generator matrix of the Markov process.}
  \item{pai0}{is the marginal probability distribution of the \eqn{m} hidden states at time zero.}
  \item{nlmp}{is logical, and determines whether the optimization mehtod nlm is used; default is \code{TRUE}.}
  \item{fortran}{is logical, and determines whether Fortran codes are used; default is \code{TRUE}.}
}
\details{
}
\value{
  \item{lamb }{a vector containing the background rates of Hawkes processes with stepwise decay.}
  \item{nu }{a vector containing the parameter with values between 0 and 1.}
  \item{eta }{a vector containing the decay rates of Hawkes processes with stepwise decay.}
  \item{Q }{the infinitesimal generator matrix of the Markov process.}
  \item{pai }{the marginal probability distribution of the \eqn{m} hidden states at time zero.}
  \item{LLo }{old log likelihood}
  \item{LLn }{new log likelihood}
}
\references{Wang, T., Bebbington, M. and Harte, D. (2012) Markov-modulated Hawkes process with stepwise decay. The Annals of the Institute of Statistical Mathematics, 64, 521-544.}
\author{Ting Wang, 2009}
\keyword{optimize}
\keyword{methods}
