      subroutine lamstar(r, n, lamb, nu, eta, ti, sums, lams, Lamst)
c      lamb is a 1*r row vector
c      nu is a 1*r row vector
c      eta is a 1*r row vector
c      ti is the occurrence times (1*(n+1)) with ti(1)=0
c      sums is an (n+1)*r matrix, which is equal to 
c        sum_{j: j< k} exp( -eta * (ti(k)-ti(j)) )
c      lams is an (n+1)*r matrix, and
c        lams(k,i) = lamb(i) + nu(i) * eta(i) * sums(k,i)
c      Lamst is an (n+1)*r*r array, and
c        Lamst(k,i,i) = lams(k,i)
       implicit none
       integer r, n, i, j, k
       double precision lamb(r), nu(r), eta(r), ti(n+1)
       double precision sums(n,r), lams(n,r), Lamst(n,r,r)
       k=2
       do while (k .LE. n)
         i=1
         do while (i .LE. r)
           j=1
           sums(k,i)=0.0
           do while (j .LE. (k-1))
             sums(k,i) = sums(k,i) + dexp(-eta(i)*(ti(k)-ti(j)))
             j=j+1
           enddo
           i=i+1
         enddo
         k=k+1
       enddo
       i=1
       do while (i .LE. r)
         sums(1,i)=0.0
         i=i+1
       enddo
       k=1
       do while (k .LE. n)
         i=1
         do while (i .LE. r)
           lams(k,i) = lamb(i) + nu(i) * eta(i) * sums(k,i)
           i=i+1
         enddo
         k=k+1
       enddo
       k=1
       do while (k .LE. n)
         i=1
         do while (i .LE. r)
           Lamst(k,i,i) = lams(k,i)
           i=i+1
         enddo
         k=k+1
       enddo
       end



       subroutine fkfun(r,n,S,Sinv,eigenv,Lamst,post,x,expmt,fk)
c      S is the non-singular matrix such that
c        Q-Lamst(ti[k]) = SDS^{-1}
c      Sinv = S^{-1}
c      post=S^{-1}Lamst(ti[k])
c      eigenv is the eigenvalues of Q-Lamst(ti[k])
c      x is the inter-event times
c      expmt is a temp for exponential of the matrix Q-Lamst(ti[k])
       implicit none
       integer r, n, l, i, j, k
       double precision S(n,r,r), Sinv(n,r,r), eigenv(n,r) 
       double precision expmt(n,r,r), fk(n,r,r)
       double precision Lamst(n,r,r), post(n,r,r), x
       l=1
       do while (l .LE. n)
         i=1
         do while (i .LE. r)
           j=1
           do while (j .LE. r)
             k=1
             fk(l,i,j)=0.0
             expmt(l,i,j)=0.0
             do while (k .LE. r)
               expmt(l,i,j)=S(l,i,k)*dexp(x*eigenv(l,k))*post(l,k,j)
               fk(l,i,j)=fk(l,i,j)+expmt(l,i,j)
               k=k+1
             enddo
             j=j+1
           enddo
           i=i+1
         enddo
         l=l+1
       enddo
       end



       subroutine Hkfun(r,n,S,Sinv,eigenv,Lamst,x,expmt,Hk)
c      S is the non-singular matrix such that
c        Q-Lamst(ti[k]) = SDS^{-1}
c      Sinv = S^{-1}
c      eigenv is the eigenvalues of Q-Lamst(ti[k])
c      x is the inter-event times
c      expmt is a temp for exponential of the matrix Q-Lamst(ti[k])
       implicit none
       integer r, n, l, i, j, k
       double precision S(n,r,r), Sinv(n,r,r), eigenv(n,r) 
       double precision expmt(n,r,r), Hk(n,r,r)
       double precision Lamst(n,r,r), x
       l=1
       do while (l .LE. n)
         i=1
         do while (i .LE. r)
           j=1
           do while (j .LE. r)
             k=1
             Hk(l,i,j)=0.0
             expmt(l,i,j)=0.0
             do while (k .LE. r)
               expmt(l,i,j)=S(l,i,k)*dexp(x*eigenv(l,k))*Sinv(l,k,j)
               Hk(l,i,j)=Hk(l,i,j)+expmt(l,i,j)
               k=k+1
             enddo
             j=j+1
           enddo
           i=i+1
         enddo
         l=l+1
       enddo
       end
